'SnapFrameHeightToInterlineMultiple.csc
'Copyright 2004 by Abe Hendin <ventura@yourspeed.com>. Public domain.
'Version 1.00
'March 5, 2004
'
'A bug in Ventura 8.559 and 10.397 causes frame height to be incorrectly set
'at certain multiples of the interline value when resizing frames with 
'interline snap turned on. For example, with a frame snapped to the bottom 
'margin and the interline snap value set to 12.00 pt (in Chapter Properties),
'resizing the frame to what should be 48.00 pt high may result in a frame 
'that's 48.01 pt high; with an 11.00 interline snap value, instead of 44.00 pt 
'you might end up with 43.99 pt. This obviously results in misfitting lines
'and a headache to resize/reposition manually.
'
'Additionally, even if the height *appears* accurate in Frame Properties, the 
'expected number of lines may still not fit. In fact, Ventura has not sized 
'the frame correctly: it is smaller than it should be, but because Ventura 
'uses tenths of a micron internally ("Ventura units"), the size difference is 
'too minute to be displayed.
'
'[An example: I dragged the top of a drawn footnote frame to 99 pt to fit nine 
'lines, but it fit only 8. The frame height in Ventura units was reported as 
'349233, or approximately 98.995119 pt, which rounds up to the 99.0 pt displayed 
'in the Frame Properties dialog. It doesn't fit 9 lines at 11.0 pt leading, 
'however. That requires a frame height of 349254 Ventura units.]
'
'I wrote this script to correct the frame height. It gets the current frame 
'height and divides it by the current chapter's interline snap setting to 
'determine the nearest multiple, then resets the frame height to a value 
'exactly divisible by that multiple. If the frame is snapped to the bottom 
'margin, it also adjusts the vertical position to keep it there.
'
'
'VERSION HISTORY:
'3/5/2004: Initial release.


#ADDFOL "C:\Program Files\Corel\Graphics8\ventura\Scripts"
#INCLUDE "VpConst.csi"
#ADDFOL "C:\Program Files\Corel\Graphics8\scripts"
#INCLUDE "scpconst.csi"

#DEFINE APP_NAME			"Snap Frame Height to Interline Multiple"
#DEFINE VERSION_STRING		" -- v1.00"
#DEFINE PAGE_LEFT			1
#DEFINE PAGE_RIGHT			2

WITHOBJECT "CorelVentura.Automation.8"
	DIM lSelectionType AS LONG, iret AS INTEGER
	DIM lFrameIndex AS LONG
	DIM lFrameHeight AS LONG, lFrameTopYPos AS LONG, lFrameBottomYPos AS LONG
	DIM sVJParaTag AS STRING, lVJCustomAmount AS LONG
	DIM lCurrPN AS LONG, bLeftPage AS BOOLEAN
	DIM lPageType AS LONG
	DIM lPageHeight AS LONG, lPageMarginBottom AS LONG
	DIM lInterline AS LONG
	DIM lFrameHeightCorrected AS LONG, lFrameYPosCorrected AS LONG
	DIM iMultiple AS INTEGER, lDiff AS LONG
	
	lSelectionType = .GetSelectionType()
	IF lSelectionType <> SELECT_FRAME THEN
		iret = MESSAGEBOX("Please select a drawn frame.", APP_NAME + VERSION_STRING, \\
			MB_STOP_ICON + MB_OK_ONLY)
		STOP
	ELSE
		lFrameIndex = .CurrentFrameIndex() 'get ref to current frame
		IF lFrameIndex < 1 THEN 'invalid frame selection
			iret = MESSAGEBOX("Please select a drawn frame.", APP_NAME + VERSION_STRING, \\
				MB_STOP_ICON + MB_OK_ONLY)
			STOP
		ENDIF
	ENDIF
	
	'collect current info
	.FormatFrameGeneralGet , lFrameTopYPos, , lFrameHeight
	.FormatChapterGeneralGet , , , sVJParaTag, lVJCustomAmount
	IF lVJCustomAmount = 0 THEN 'interline snap is based on paratag interline
		.FormatParaTagBegin sVJParaTag
			.FormatParaSpacingGet , , lInterline 'get paratag interline spacing
		.FormatParaTagEnd
	ELSE
		lInterline = lVJCustomAmount
	ENDIF
	.FrameFirst TRUE 'select base page
	lCurrPN = .CurrentPageNumber(bLeftPage,2) 'get page side for accurate margin retrieval
	IF bLeftPage THEN
		lPageType = PAGE_LEFT
	ELSE
		lPageType = PAGE_RIGHT
	ENDIF
	.FormatFrameGeneralGet , , , lPageHeight
	.FormatFrameMarginsInsideGet lPageType, , , , lPageMarginBottom
	
	'make calculations
	iMultiple = INT((lFrameHeight / lInterline) + .5) 'round off to nearest exact multiple
	lFrameHeightCorrected = lInterline * iMultiple 'set corrected frame height
	lFrameBottomYPos = lFrameTopYPos + lFrameHeight 'to check whether snapped to bottom margin
	
	'if frame is snapped to bottom margin, keep it there (resize up)
	IF lFrameBottomYPos = lPageHeight - lPageMarginBottom THEN 
		lDiff = lFrameHeightCorrected - lFrameHeight
		lFrameYPosCorrected = lFrameTopYPos - lDiff
	ELSE
		lFrameYPosCorrected = lFrameTopYPos
	ENDIF
	
	'apply new height/position
	.FrameFirst FALSE, lFrameIndex 'reselect frame
	.FormatFrameGeneral , lFrameYPosCorrected, , lFrameHeightCorrected
END WITHOBJECT
